@@ -7,150 +7,85 @@ import os
 import sys
 import re
 import json, csv
+import yaml
 import pandas as pd
-from datetime import datetime
-from typing import Dict, List, Tuple, Any, Optional
-from datasets import load_dataset
+import warnings
+import subprocess
+import datetime
+from typing import Dict, List, Tuple, Any, Optional, Union
+try:
+    # Prefer the HuggingFace 'datasets' library. If a conflicting local module existed
+    # (e.g., a folder named 'datasets'), it could shadow the external package. That
+    # folder has been renamed to 'eval_datasets' to avoid ImportError.
+    from datasets import load_dataset  # type: ignore
+except Exception as _import_err:  # pragma: no cover
+    raise ImportError(
+        f"Failed to import 'load_dataset' from HuggingFace datasets library: {_import_err}. "
+        "Ensure 'datasets' is installed (pip install datasets) and no local 'datasets' package shadows it."
+    )
 import numpy as np
 
+# Suppress Pydantic warnings from dependencies (TRL/transformers)
+warnings.filterwarnings("ignore", category=UserWarning, module="pydantic._internal._generate_schema")
+warnings.filterwarnings("ignore", message=".*'repr' attribute.*has no effect.*")
+
 # Add project root to path
 project_root = os.path.abspath(os.path.join(os.path.dirname(__file__), '..'))
 if project_root not in sys.path:
     sys.path.insert(0, project_root)
 
 # Import existing modules
-from core.config import ExperimentConfig
-from inference.inference import run_inference, load_model_and_tokenizer, generate_response, generate_response_by_api
+from core.config import ExperimentConfig, EvaluationConfig
+from inference.inference import run_inference, load_model_and_tokenizer, generate_response
+from utils.api_utils import generate_response_by_api
+from utils.recipe_overrides import apply_overrides_to_recipe, load_recipe_from_yaml
+
+# Import dataset-specific evaluation utilities
+from evaluations.eval_datasets import mmlu, gsm8k
 
 
-def extract_multiple_choice_answer(text, choice_labels=None):
+def extract_predicted_answer(text, dataset_name, choice_labels=None):
     """
-    Extract the first character from text if it matches a choice label.
+    Extract predicted answer from model response based on dataset.
     
     Args:
         text: Model response text
-        choice_labels: List of valid choice labels
-    
-    Returns:
-        Extracted choice label or None if not found
-    """
-    if not text:
-        return None
-    
-    if choice_labels is None:
-        choice_labels = ["A", "B", "C", "D", "E", "F", "G", "H", "I", "J"]
-
-    text_upper = text.upper()
-
-    # Pattern 0: JSON format, e.g., "answer": "B" or “answer”: “B”
-    pattern0 = r'[“"]ANSWER[”"]\s*:\s*[“"]([' + ''.join(choice_labels) + r'])[”"]'
-
-    match = re.search(pattern0, text_upper)
-    if match:
-        return match.group(1)
-    
-    # Pattern 1: "The best/final answer/option is A" or "The answer/option is A"
-    pattern1 = r'(?:THE\s+)?(?:BEST|FINAL)?\s*(?:ANSWER|OPTION)\s+IS\s+([' + ''.join(choice_labels) + r'])'
-    match = re.search(pattern1, text_upper)
-    if match:
-        return match.group(1)
-    
-    # Pattern 2: "Answer: A" (with optional additional text)
-    pattern2 = r'ANSWER:\s*([' + ''.join(choice_labels) + r'])'
-    match = re.search(pattern2, text_upper)
-    if match:
-        return match.group(1)
-
-    # Pattern 3 for "option/answer C is the best/final answer/option"
-    pattern3 = r'(?:OPTION|ANSWER)\s+([' + ''.join(choice_labels) + r'])\s+IS\s+(?:THE\s+)?(?:BEST|FINAL)?\s*(?:ANSWER|OPTION)'
-    match = re.search(pattern3, text_upper)
-    if match:
-        return match.group(1)
-
-    # Pattern 4: "A."
-    first_char = text.strip().upper()[0]
-    
-    return first_char if first_char in choice_labels else None
-
-
-def convert_answer_index_to_letter(answer_index, choice_labels=None):
-    """
-    Convert numerical answer index to choice letter.
-    
-    Args:
-        answer_index: Numerical index (0, 1, 2, 3...)
-        choice_labels: List of choice labels
-    
-    Returns:
-        Choice letter (A, B, C, D...)
-    """
-    if choice_labels is None:
-        choice_labels = ["A", "B", "C", "D", "E", "F", "G", "H", "I", "J"]
-    
-    try:
-        index = int(answer_index)
-        if 0 <= index < len(choice_labels):
-            return choice_labels[index]
-    except (ValueError, TypeError):
-        pass
-    
-    return None
-
-
-def extract_predicted_answer(text, output_type="numerical", choice_labels=None):
-    """
-    Extract predicted answer from model response based on output type.
-    
-    Args:
-        text: Model response text
-        output_type: Type of answer format ("numerical", "multiple_choice")
+        dataset_name: Name of the dataset being evaluated
         choice_labels: List of choice labels for multiple choice
     
     Returns:
         Extracted answer or None if not found
     """
-    if output_type == "multiple_choice":
-        return extract_multiple_choice_answer(text, choice_labels)
-    else:
-        # Original numerical extraction logic
-        regex_pattern = "(-?[$0-9.,]{2,})|(-?[0-9]+)"
-        regexes_to_ignore = [",", "\\$", "(?s).*#### ", "\\.$"]
-        
-        match = re.findall(regex_pattern, text)
-        if match:
-            match = match[-1]
-            if isinstance(match, tuple):
-                match = [m for m in match if m][0]
-            text = match.strip()
-
-            for regex in regexes_to_ignore:
-                text = re.sub(regex, "", text)
-            return text
-        else:
-            return None
+    if dataset_name == "cais/mmlu":
+        return mmlu.extract_predicted_answer(text, choice_labels)
+    if dataset_name == "openai/gsm8k":
+        return gsm8k.extract_predicted_answer(text)
+    
+    print(f"{dataset_name} does not support.")
+    return None
 
 
-def extract_ground_truth(text, output_type="numerical", choice_labels=None):
+def extract_ground_truth(text, dataset_name, choice_labels=None):
     """
-    Extract ground truth answer based on output type.
+    Extract ground truth answer based on dataset.
     
     Args:
         text: Ground truth text or index
-        output_type: Type of answer format
+        dataset_name: Name of the dataset being evaluated
         choice_labels: List of choice labels for multiple choice
     
     Returns:
         Processed ground truth answer
     """
-    if output_type == "multiple_choice":
-        # For MMLU, ground truth is typically a numerical index
-        return convert_answer_index_to_letter(text, choice_labels)
-    else:
-        # Original logic for numerical answers
-        return text.split('####')[-1].strip()
+    if dataset_name == "cais/mmlu":
+        # For MMLU, ground truth is a numerical index
+        return mmlu.extract_ground_truth(text, choice_labels)
+    if dataset_name == "openai/gsm8k":
+        return gsm8k.extract_ground_truth(text)
+    return text
 
 
-def load_evaluation_dataset(dataset_name: str, split: str = "test", subset: str = None) -> pd.DataFrame:
+def load_evaluation_dataset(dataset_name: str, split: str = "test", subset: Optional[str] = None) -> pd.DataFrame:
     """
     Load evaluation dataset and return as DataFrame.
     
@@ -183,7 +118,7 @@ def load_evaluation_dataset(dataset_name: str, split: str = "test", subset: str
     return df
 
 
-def run_inference_for_evaluation(config: ExperimentConfig, debug: bool = False) -> pd.DataFrame:
+def run_inference_for_evaluation(config: Union[ExperimentConfig, EvaluationConfig], debug: bool = False) -> pd.DataFrame:
     """
     Run inference using the existing inference system and return results.
     
@@ -199,14 +134,15 @@ def run_inference_for_evaluation(config: ExperimentConfig, debug: bool = False)
     run_inference(config, debug)
     
     # Load the inference results
-    results_df = pd.read_csv(config.output_file)
-    print(f"Loaded {len(results_df)} inference results from {config.output_file}")
+    # Both InferenceConfig and EvaluationConfig define output_file attribute.
+    results_df = pd.read_csv(config.output_file)  # type: ignore[attr-defined]
+    print(f"Loaded {len(results_df)} inference results from {config.output_file}")  # type: ignore[attr-defined]
     
     return results_df
 
 
 def calculate_accuracy_metrics(predictions: List[Optional[str]], 
-                             ground_truths: List[str]) -> Dict[str, float]:
+                             ground_truths: List[str]) -> Tuple[Dict[str, float], List[Dict[str, Any]]]:
     """
     Calculate accuracy metrics by comparing predictions with ground truths.
     
@@ -265,8 +201,43 @@ def calculate_accuracy_metrics(predictions: List[Optional[str]],
     return metrics, detailed_results
 
 
-def run_comprehensive_evaluation(config_path: str, 
-                                debug: bool = False) -> Dict[str, Any]:
+def load_eval_recipe_with_overrides(recipe_path: str, overrides: List[str]):
+    """Load evaluation recipe from file and/or command-line arguments.
+    
+    Priority (highest to lowest):
+    1. Command-line overrides
+    2. Recipe file values
+    3. Default values
+    """
+    if not recipe_path:
+        raise ValueError("recipe_path is required")
+    
+    # Load base recipe from YAML
+    recipe_dict = load_recipe_from_yaml(recipe_path)
+    
+    # Apply command-line overrides
+    if overrides:
+        recipe_dict = apply_overrides_to_recipe(recipe_dict, overrides)
+    
+    # Write temporary recipe file with overrides applied
+    import tempfile
+    with tempfile.NamedTemporaryFile(mode='w', suffix='.yaml', delete=False) as tmp_file:
+        yaml.dump(recipe_dict, tmp_file)
+        tmp_recipe_path = tmp_file.name
+    
+    try:
+        # Load using existing config loader
+        config = ExperimentConfig.load_eval_config(tmp_recipe_path)
+    finally:
+        # Clean up temporary file
+        os.unlink(tmp_recipe_path)
+    
+    return config
+
+
+def run_comprehensive_evaluation(recipe_path: str, 
+                                debug: bool = False,
+                                overrides: Optional[List[str]] = None) -> Dict[str, Any]:
     """
     Run comprehensive evaluation by:
     1. Loading evaluation dataset
@@ -275,19 +246,19 @@ def run_comprehensive_evaluation(config_path: str,
     4. Calculating accuracy metrics
     
     Args:
-        config_path: Path to evaluation configuration file
+        recipe_path: Path to evaluation recipe file
         debug: Enable debug logging
+        overrides: List of recipe overrides
         
     Returns:
         Dictionary containing evaluation results
     """
     print("Starting comprehensive evaluation...")
-    print(f"Config: {config_path}")
+    print(f"Recipe: {recipe_path}")
     
     try:
-        # Load configuration to get dataset name and other settings
-        config = ExperimentConfig.load_eval_config(config_path)
-        output_type = getattr(config, 'output_type', 'numerical')
+        # Load recipe to get dataset name and other settings with overrides
+        config = load_eval_recipe_with_overrides(recipe_path, overrides or [])
         choice_labels = getattr(config, 'choice_labels', None)
         dataset_subset = getattr(config, 'dataset_subset', None)
         evaluation_split = getattr(config, 'dataset_split', 'test')
@@ -308,11 +279,12 @@ def run_comprehensive_evaluation(config_path: str,
         
         # Extract predicted answers
         print("Extracting predicted answers...")
+        response_col = getattr(config, 'response_column', 'response')
         predicted_answers = []
-        for response in inference_results['response']:
+        for response in inference_results[response_col]:
             pred_answer = extract_predicted_answer(
                 response, 
-                output_type=output_type,
+                dataset_name=evaluation_dataset_name,
                 choice_labels=choice_labels
             )
             predicted_answers.append(pred_answer)
@@ -323,7 +295,7 @@ def run_comprehensive_evaluation(config_path: str,
         for truth_text in eval_dataset[ground_truth_column]:
             gt_answer = extract_ground_truth(
                 str(truth_text), 
-                output_type=output_type,
+                dataset_name=evaluation_dataset_name,
                 choice_labels=choice_labels
             )
             ground_truth_answers.append(gt_answer)
@@ -344,9 +316,9 @@ def run_comprehensive_evaluation(config_path: str,
         print(f"Extraction Success Rate: {metrics['extraction_success_rate']:.4f} ({metrics['valid_predictions']}/{metrics['total_examples']})")
         
         # Save detailed results
-        timestamp = datetime.now().strftime("%Y%m%d_%H%M%S")
+        timestamp = datetime.datetime.now().strftime("%Y%m%d_%H%M%S")
         evaluation_results = {
-            'config_path': config_path,
+            'recipe_path': recipe_path,
             'evaluation_dataset': evaluation_dataset_name,
             'evaluation_split': evaluation_split,
             'ground_truth_column': ground_truth_column,
@@ -401,7 +373,7 @@ def run_comprehensive_evaluation(config_path: str,
 def calculate_accuracy_metrics_with_dataset_columns(predictions: List[Optional[str]], 
                                                    ground_truths: List[str],
                                                    eval_dataset: pd.DataFrame,
-                                                   dataset_columns: List[str]) -> Dict[str, float]:
+                                                   dataset_columns: List[str]) -> Tuple[Dict[str, float], List[Dict[str, Any]]]:
     """
     Calculate accuracy metrics by comparing predictions with ground truths,
     and include dataset columns in the detailed results.
@@ -474,38 +446,102 @@ def calculate_accuracy_metrics_with_dataset_columns(predictions: List[Optional[s
 
 def parse_args():
     """Parse command line arguments."""
-    parser = argparse.ArgumentParser(description="Run comprehensive model evaluation")
+    parser = argparse.ArgumentParser(
+        description="Run comprehensive model evaluation",
+        formatter_class=argparse.RawDescriptionHelpFormatter,
+        epilog="""
+Examples:
+  # Using recipe file:
+  fai-rl-eval --recipe recipes/evaluation/mmlu/llama3_3B.yaml
+  
+  # Mix recipe file with overrides:
+  fai-rl-eval --recipe recipe.yaml model_path='./my_model' temperature=0.7
+  
+  # Override evaluation parameters:
+  fai-rl-eval --recipe recipe.yaml dataset_split='validation' max_new_tokens=256
+"""
+    )
     parser.add_argument(
-        "--config",
+        "--recipe",
         type=str,
-        required=True,
-        help="Path to inference configuration YAML file"
+        default=None,
+        help="Path to evaluation recipe YAML file"
     )
     parser.add_argument(
         "--debug",
         action="store_true",
         help="Enable debug mode with verbose logging"
     )
+    parser.add_argument(
+        "--nohup",
+        action="store_true",
+        help="Run evaluation in background with nohup (output redirected to evaluation_<timestamp>.log)"
+    )
+    parser.add_argument(
+        "overrides",
+        nargs="*",
+        help="Recipe overrides in key=value format (e.g., model_path='./output' dataset_split='test')"
+    )
     
-    return parser.parse_args()
+    args = parser.parse_args()
+    
+    # Add this check: if no arguments provided at all, show help and exit
+    if len(sys.argv) == 1:
+        parser.print_help(sys.stderr)
+        sys.exit(1)
+    
+    return args
 
 
 def main():
     """Main evaluation function."""
     args = parse_args()
     
+    # Handle nohup mode
+    if args.nohup:
+        # Generate log filename with timestamp
+        timestamp = datetime.datetime.now().strftime("%Y%m%d_%H%M%S")
+        log_file = f"evaluation_{timestamp}.log"
+        
+        print(f"Running evaluation in background with nohup. Output will be saved to: {log_file}")
+        
+        # Build command to run the script without --nohup
+        script_path = os.path.abspath(__file__)
+        cmd_args = [sys.executable, script_path, "--recipe", args.recipe]
+        if args.debug:
+            cmd_args.append("--debug")
+        
+        # Add overrides to command
+        if args.overrides:
+            cmd_args.extend(args.overrides)
+        
+        # Prepare nohup command: nohup <command> > log_file 2>&1 &
+        cmd_str = " ".join(cmd_args) + f" > {log_file} 2>&1 &"
+        full_cmd = f"nohup {cmd_str}"
+        
+        print(f"Executing: {full_cmd}")
+        
+        # Execute with shell to handle redirection and background
+        result = subprocess.call(full_cmd, shell=True)
+        
+        if result == 0:
+            print(f"Evaluation started in background. Monitor progress with: tail -f {log_file}")
+        
+        return result
+    
     # Create necessary directories
     os.makedirs("logs", exist_ok=True)
     os.makedirs("outputs", exist_ok=True)
     
     print("Starting Comprehensive Model Evaluation")
-    print(f"Config: {args.config}")
+    print(f"Recipe: {args.recipe}")
     print()
     
     try:
         results = run_comprehensive_evaluation(
-            config_path=args.config,
-            debug=args.debug
+            recipe_path=args.recipe,
+            debug=args.debug,
+            overrides=args.overrides
         )
         
         return results
