@@ -7,20 +7,36 @@ import torch
 import os
 import json, csv
 import sys
-import pandas as pd
-import requests
-import re
-from typing import Dict, Any, Union
-from transformers import AutoTokenizer, AutoModelForCausalLM
-from datasets import load_dataset
-from pathlib import Path
-from peft import PeftModel, PeftConfig
 
+# Disable fast tokenizer conversion to avoid tiktoken issues
+os.environ["TOKENIZERS_PARALLELISM"] = "false"
+
+# Add project root to path BEFORE importing local modules
 project_root = os.path.abspath(os.path.join(os.path.dirname(__file__), '..'))
 if project_root not in sys.path:
     sys.path.insert(0, project_root)
 
+import yaml
+import pandas as pd
+import requests
+import re
+import warnings
+import subprocess
+import datetime
+from typing import Dict, Any, Union
+from transformers import AutoTokenizer, AutoModelForCausalLM
+from datasets import load_dataset
+from utils.api_utils import generate_response_by_api
+
+# Suppress Pydantic warnings from dependencies (TRL/transformers)
+warnings.filterwarnings("ignore", category=UserWarning, module="pydantic._internal._generate_schema")
+warnings.filterwarnings("ignore", message=".*'repr' attribute.*has no effect.*")
+from pathlib import Path
+from peft import PeftModel, PeftConfig
+
 from core.config import ExperimentConfig
+from utils.config_validation import validate_api_config
+from utils.recipe_overrides import apply_overrides_to_recipe, load_recipe_from_yaml
 
 
 def format_multiple_choice_for_inference(choices, choice_labels=None):
@@ -81,8 +97,8 @@ def format_template_prompt(template, example, config):
     # Create a copy of the example for formatting
     format_dict = example.copy()
     
-    # Handle multiple choice formatting if needed
-    if hasattr(config, 'output_type') and config.output_type == "multiple_choice":
+    # Handle multiple choice formatting if needed (for MMLU dataset)
+    if hasattr(config, 'dataset_name') and config.dataset_name == "cais/mmlu":
         if 'choices' in format_dict:
             choice_labels = getattr(config, 'choice_labels', None)
             formatted_choices = format_multiple_choice_for_inference(
@@ -104,57 +120,113 @@ def format_template_prompt(template, example, config):
 
 def parse_args():
     """Parse command line arguments."""
-    parser = argparse.ArgumentParser(description="Run model inference")
+    parser = argparse.ArgumentParser(
+        description="Run model inference",
+        formatter_class=argparse.RawDescriptionHelpFormatter,
+        epilog="""
+Examples:
+  # Using recipe file:
+  fai-rl-inference --recipe recipes/inference/llama3_3B.yaml
+  
+  # Mix recipe file with overrides:
+  fai-rl-inference --recipe recipe.yaml model_path='./my_model' temperature=0.7
+  
+  # Override inference parameters:
+  fai-rl-inference --recipe recipe.yaml max_new_tokens=512 do_sample=True
+"""
+    )
     parser.add_argument(
-        "--config",
+        "--recipe",
         type=str,
-        required=True,
-        help="Path to inference configuration YAML file"
+        default=None,
+        help="Path to inference recipe YAML file"
     )
     parser.add_argument(
         "--debug",
         action="store_true",
         help="Enable debug mode with verbose logging"
     )
+    parser.add_argument(
+        "--nohup",
+        action="store_true",
+        help="Run inference in background with nohup (output redirected to inference_<timestamp>.log)"
+    )
+    parser.add_argument(
+        "overrides",
+        nargs="*",
+        help="Recipe overrides in key=value format (e.g., model_path='./output' temperature=0.7)"
+    )
     
-    return parser.parse_args()
+    args = parser.parse_args()
+    
+    # Add this check: if no arguments provided at all, show help and exit
+    if len(sys.argv) == 1:
+        parser.print_help(sys.stderr)
+        sys.exit(1)
+    
+    return args
 
 
 def load_model_and_tokenizer(config):
     """Load model and tokenizer based on config."""
-    model_path = config.model_path
+    # Support both model_path (local) and model (HuggingFace hub)
+    if hasattr(config, 'model_path') and config.model_path:
+        model_identifier = config.model_path
+        is_local = True
+    elif hasattr(config, 'model') and config.model:
+        model_identifier = config.model
+        is_local = False
+    else:
+        raise ValueError("Either model_path or model must be specified in config")
     
-    # Handle relative paths by prepending current working directory
-    if not os.path.isabs(model_path):
-        model_path = os.path.join(os.getcwd(), model_path)
+    # Handle relative paths for local models
+    if is_local and not os.path.isabs(model_identifier):
+        model_identifier = os.path.join(os.getcwd(), model_identifier)
     
-    print(f"Loading model from: {model_path}")
+    print(f"Loading model from: {model_identifier}")
     
-    if not os.path.exists(model_path):
-        raise FileNotFoundError(f"Model path does not exist: {model_path}")
+    # Check if path exists for local models
+    if is_local and not os.path.exists(model_identifier):
+        raise FileNotFoundError(f"Model path does not exist: {model_identifier}")
     
-    # Check if this is a PEFT checkpoint
-    adapter_config_path = os.path.join(model_path, "adapter_config.json")
-    is_peft_checkpoint = os.path.exists(adapter_config_path)
+    # Check if this is a PEFT checkpoint (only for local models)
+    is_peft_checkpoint = False
+    if is_local:
+        adapter_config_path = os.path.join(model_identifier, "adapter_config.json")
+        is_peft_checkpoint = os.path.exists(adapter_config_path)
     
     if is_peft_checkpoint:
         print("Detected PEFT/LoRA checkpoint, loading base model first...")
         
         # Load the PEFT config to get the base model name
-        peft_config = PeftConfig.from_pretrained(model_path)
+        peft_config = PeftConfig.from_pretrained(model_identifier)
         base_model_name = peft_config.base_model_name_or_path
         
         print(f"Base model: {base_model_name}")
         
-        # Load tokenizer from checkpoint
-        tokenizer = AutoTokenizer.from_pretrained(model_path)
+        # Load tokenizer from base model instead of checkpoint to avoid conversion issues
+        print("Loading tokenizer from base model...")
+        try:
+            tokenizer = AutoTokenizer.from_pretrained(
+                base_model_name,
+                use_fast=False,
+                legacy=False
+            )
+        except Exception as e:
+            print(f"Warning: Failed to load tokenizer with use_fast=False, trying with trust_remote_code=True: {e}")
+            tokenizer = AutoTokenizer.from_pretrained(
+                base_model_name,
+                use_fast=False,
+                trust_remote_code=True,
+                legacy=False
+            )
         
         # Set the pad token if it's not already set
         if tokenizer.pad_token is None:
             tokenizer.pad_token = tokenizer.eos_token
         
         # Add the special pad token to match training setup (PPO adds "[PAD]" token)
-        if "[PAD]" not in tokenizer.get_vocab():    # TODO:: add this special tokens for all algorithms
+        if "[PAD]" not in tokenizer.get_vocab():
             tokenizer.add_special_tokens({"pad_token": "[PAD]"})
             print(f"Added [PAD] token to tokenizer. New vocab size: {len(tokenizer)}")
         
@@ -173,18 +245,31 @@ def load_model_and_tokenizer(config):
         
         # Now load the PEFT adapter
         print("Loading PEFT adapter...")
-        model = PeftModel.from_pretrained(model, model_path)
+        model = PeftModel.from_pretrained(model, model_identifier)
         
         # Merge adapter weights for faster inference
         print("Merging adapter weights...")
         model = model.merge_and_unload()
         
     else:
-        # Regular model loading (non-PEFT)
-        print("Loading regular (non-PEFT) model...")
+        # Regular model loading (non-PEFT) - can be local or from HuggingFace hub
+        print(f"Loading regular model from {'local path' if is_local else 'HuggingFace hub'}...")
         
-        # Load tokenizer
-        tokenizer = AutoTokenizer.from_pretrained(model_path)
+        # Load tokenizer (use slow tokenizer to avoid tiktoken conversion issues)
+        try:
+            tokenizer = AutoTokenizer.from_pretrained(
+                model_identifier,
+                use_fast=False,
+                legacy=False
+            )
+        except Exception as e:
+            print(f"Warning: Failed to load tokenizer with use_fast=False, trying with trust_remote_code=True: {e}")
+            tokenizer = AutoTokenizer.from_pretrained(
+                model_identifier,
+                use_fast=False,
+                trust_remote_code=True,
+                legacy=False
+            )
         
         # Set the pad token if it's not already set
         if tokenizer.pad_token is None:
@@ -192,7 +277,7 @@ def load_model_and_tokenizer(config):
         
         # Load the model
         model = AutoModelForCausalLM.from_pretrained(
-            model_path,
+            model_identifier,
             torch_dtype=torch.bfloat16,
             device_map="auto"
         )
@@ -232,135 +317,25 @@ def generate_response(model, tokenizer, prompt: str, config):
     
     return response_text
 
-def _get_api_endpoint(model: str) -> str:
-    """Determine the appropriate API endpoint based on model type and name."""
-    # ST3 models that require the sitetest3 endpoint
-    st3_models = [
-        'Qwen/Qwen3-235B-A22B-Instruct-2507', 
-        'openai/gpt-oss-120b', 
-        'openai/o3', 
-        'openai/gpt-4o', 
-        'anthropic/claude-opus-4'
-    ]
-    
-    if model.startswith("google/"):
-        model_name = model.split("/")[-1]
-        return f"https://apis.sitetest3.simulpong.com/ml-gateway-service/gemini/v1beta/models/{model_name}:generateContent"
-    elif model in st3_models:
-        return "https://apis.sitetest3.simulpong.com/ml-gateway-service/v1/chat/completions"
-    else:
-        # Default ST1 endpoint
-        return 'https://snc2-apis.sitetest1.simulpong.com/ml-gateway-service/v1/chat/completions'
-
-
-def _build_google_request_data(prompt: str, config) -> dict:
-    """Build request data for Google/Gemini models."""
-    return {
-        "contents": {
-            "role": "user",
-            "parts": [{"text": prompt}]
-        },
-        "generationConfig": {
-            "maxOutputTokens": config.max_new_tokens
-        }
-    }
-
-
-def _build_openai_request_data(prompt: str, config) -> dict:
-    """Build request data for OpenAI models."""
-    return {
-        "model": config.model,
-        "messages": [{"content": prompt, "role": "user"}]
-    }
-
-
-def _build_default_request_data(prompt: str, config) -> dict:
-    """Build request data for other models (Anthropic, etc.)."""
-    return {
-        "model": config.model,
-        "max_tokens": config.max_new_tokens,
-        "temperature": config.temperature,
-        "messages": [{"content": prompt, "role": "user"}]
-    }
-
-
-def _make_api_request(url: str, headers: dict, data: dict, model: str) -> requests.Response:
-    """Make the HTTP request to the API endpoint."""
-    if model.startswith("google/"):
-        # Google API requires data to be JSON string in body
-        return requests.post(url, headers=headers, data=json.dumps(data))
-    else:
-        # Other APIs can use json parameter
-        return requests.post(url, headers=headers, json=data)
-
-
-def _parse_api_response(response_json: dict, model: str) -> str:
-    """Extract the response text from the API response JSON."""
-    try:
-        if model.startswith("google/"):
-            return response_json['candidates'][0]['content']['parts'][0]['text']
-        else:
-            return response_json['choices'][0]['message']['content']
-    except (KeyError, IndexError, TypeError):
-        return ""
-
-
-def _validate_api_key(api_key: str) -> None:
-    """Validate that the API key is not a placeholder."""
-    if api_key == "YOUR_MLP_API_KEY":
-        raise ValueError(
-            "Error: mlp_api_key is still set to the placeholder 'YOUR_MLP_API_KEY'. "
-            "Please replace it with your actual API key in the configuration file."
-        )
-
-
-def generate_response_by_api(
-    prompt: str,
-    config
-) -> Union[Dict[str, Any], str]:
-    """Generate response using API-based inference."""
-    _validate_api_key(config.mlp_api_key)
-    
-    try:
-        # Get the appropriate API endpoint
-        url = _get_api_endpoint(config.model)
-        
-        # Set up headers
-        headers = {
-            "Content-Type": "application/json",
-            "Authorization": config.mlp_api_key
-        }
-        
-        # Build request data based on model type
-        if config.model.startswith("google/"):
-            data = _build_google_request_data(prompt, config)
-        elif config.model.startswith("openai/"):
-            data = _build_openai_request_data(prompt, config)
-        else:
-            data = _build_default_request_data(prompt, config)
-        
-        # Make the API request
-        response = _make_api_request(url, headers, data, config.model)
-        response.raise_for_status()
-        
-        # Parse and return the response
-        response_json = response.json()
-        return _parse_api_response(response_json, config.model)
-        
-    except requests.exceptions.RequestException as e:
-        return ""
-
 
 def run_inference(config, debug=False):
     """Run inference on the specified dataset."""
     # Determine if we should use API or local model
-    use_api = (config.model is not None) and (config.mlp_api_key is not None)
+    # API requires both model and api_key
+    use_api = (hasattr(config, 'model') and config.model is not None) and \
+              (hasattr(config, 'api_key') and config.api_key is not None)
     
     if use_api:
         print(f"Using API inference with model: {config.model}")
         model, tokenizer = None, None
     else:
-        print("Using local model inference")
+        # Local inference - supports both model_path (local fine-tuned) and model (HuggingFace vanilla)
+        if hasattr(config, 'model_path') and config.model_path:
+            print(f"Using local fine-tuned model from: {config.model_path}")
+        elif hasattr(config, 'model') and config.model:
+            print(f"Using vanilla HuggingFace model: {config.model}")
+        else:
+            raise ValueError("Either model_path or model must be specified for local inference")
         model, tokenizer = load_model_and_tokenizer(config)
 
     # Load dataset
@@ -413,8 +388,9 @@ def run_inference(config, debug=False):
                 print(f"{response}")
                 print(f"{'='*50}\n")
             
-            # Store the result with flattened used_columns
-            result = {'response': response}
+            # Store the result with configurable response column name
+            response_col = getattr(config, 'response_column', 'response')
+            result = {response_col: response}
             
             # Flatten used_columns into separate columns
             for col in config.dataset_columns:
@@ -440,14 +416,25 @@ def run_inference(config, debug=False):
     print(f"\nResults saved to: {config.output_file}")
     print(f"Processed {len(results)} examples successfully")
     
+    # Determine model info for summary
+    if use_api:
+        model_info = config.model
+        inference_type = 'api'
+    elif hasattr(config, 'model_path') and config.model_path:
+        model_info = config.model_path
+        inference_type = 'local_finetuned'
+    else:
+        model_info = config.model
+        inference_type = 'huggingface_vanilla'
+    
     # Create summary
     summary = {
         'total_examples': len(data_split),
         'successful_examples': len(results),
         'failed_examples': len(data_split) - len(results),
         'config': config.to_dict(),
-        'inference_type': 'api' if use_api else 'local',
-        'model_info': config.model if use_api else config.model_path,
+        'inference_type': inference_type,
+        'model_info': model_info,
         'dataset_name': config.dataset_name,
         'dataset_columns_used': config.dataset_columns,
         'system_prompt': config.system_prompt
@@ -461,24 +448,96 @@ def run_inference(config, debug=False):
     print(f"Summary saved to: {summary_file}")
 
 
+def load_inference_recipe_with_overrides(args):
+    """Load inference recipe from file and/or command-line arguments.
+    
+    Priority (highest to lowest):
+    1. Command-line overrides
+    2. Recipe file values
+    3. Default values
+    """
+    if not args.recipe:
+        raise ValueError("--recipe argument is required")
+    
+    # Load base recipe from YAML
+    recipe_dict = load_recipe_from_yaml(args.recipe)
+    
+    # Apply command-line overrides
+    if args.overrides:
+        recipe_dict = apply_overrides_to_recipe(recipe_dict, args.overrides)
+    
+    # Write temporary recipe file with overrides applied
+    import tempfile
+    with tempfile.NamedTemporaryFile(mode='w', suffix='.yaml', delete=False) as tmp_file:
+        yaml.dump(recipe_dict, tmp_file)
+        tmp_recipe_path = tmp_file.name
+    
+    try:
+        # Load using existing config loader
+        config = ExperimentConfig.load_inference_config(tmp_recipe_path)
+    finally:
+        # Clean up temporary file
+        os.unlink(tmp_recipe_path)
+    
+    return config
+
+
 def main():
     """Main inference function."""
     global args
     args = parse_args()
     
-    # Load configuration
-    config = ExperimentConfig.load_inference_config(args.config)
+    # Handle nohup mode
+    if args.nohup:
+        # Generate log filename with timestamp
+        timestamp = datetime.datetime.now().strftime("%Y%m%d_%H%M%S")
+        log_file = f"inference_{timestamp}.log"
+        
+        print(f"Running inference in background with nohup. Output will be saved to: {log_file}")
+        
+        # Build command to run the script without --nohup
+        script_path = os.path.abspath(__file__)
+        cmd_args = [sys.executable, script_path, "--recipe", args.recipe]
+        if args.debug:
+            cmd_args.append("--debug")
+        
+        # Add overrides to command
+        if args.overrides:
+            cmd_args.extend(args.overrides)
+        
+        # Prepare nohup command: nohup <command> > log_file 2>&1 &
+        cmd_str = " ".join(cmd_args) + f" > {log_file} 2>&1 &"
+        full_cmd = f"nohup {cmd_str}"
+        
+        print(f"Executing: {full_cmd}")
+        
+        # Execute with shell to handle redirection and background
+        result = subprocess.call(full_cmd, shell=True)
+        
+        if result == 0:
+            print(f"Inference started in background. Monitor progress with: tail -f {log_file}")
+        
+        return result
     
-    # Check if we should use API-based inference
-    use_api = hasattr(config, 'mlp_api_key') and config.mlp_api_key
+    # Load recipe with overrides
+    config = load_inference_recipe_with_overrides(args)
+    
+    # Determine inference type
+    use_api = hasattr(config, 'api_key') and config.api_key and \
+              hasattr(config, 'model') and config.model
     
     print("Starting inference with the following configuration:")
     if use_api:
         print(f"  Model (API): {config.model}")
         print(f"  Inference type: API-based")
-    else:
+    elif hasattr(config, 'model_path') and config.model_path:
         print(f"  Model path: {config.model_path}")
-        print(f"  Inference type: Local model")
+        print(f"  Inference type: Local fine-tuned model")
+    elif hasattr(config, 'model') and config.model:
+        print(f"  Model: {config.model}")
+        print(f"  Inference type: HuggingFace vanilla model")
+    else:
+        raise ValueError("Either model_path or model must be specified in config")
     
     print(f"  Dataset: {config.dataset_name}")
     print(f"  Dataset columns: {config.dataset_columns}")
