@@ -1,5 +1,6 @@
 """Template modules for dataset formatting."""
 from .gsm8k_template import GSM8KTemplate
 from .openmathinstruct_template import OpenMathInstructTemplate
+from .subjective_template import UnverifiableDomainTemplate
 
-__all__ = ['GSM8KTemplate', 'OpenMathInstructTemplate']
+__all__ = ['GSM8KTemplate', 'OpenMathInstructTemplate', 'UnverifiableDomainTemplate']
